import itertools

from marshmallow import ValidationError

from . import comic, exceptions


class ComicsList():
    def __init__(self, response):
        self.comics = []
        self.response = response

        schema = comic.ComicSchema()
        for comic_dict in response['data']['results']:
            try:
                result = schema.load(comic_dict)
            except ValidationError as error:
                raise exceptions.ApiError(error)

            self.comics.append(result)

    def __iter__(self):
        return iter(self.comics)

    def __len__(self):
        return len(self.comics)

    def __getitem__(self, index):
        try:
            return next(itertools.islice(self.comics, index, index+1))
        except TypeError:
            return list(itertools.islice(
                self.comics, index.start, index.stop, index.step))



import unittest

import marvelous
from marvelous.exceptions import AuthenticationError


class TestInit(unittest.TestCase):
    def setUp(self):
        pass

    def test_api(self):
        with self.assertRaises(AuthenticationError):
            marvelous.api()

        with self.assertRaises(AuthenticationError):
            marvelous.api(private_key="Something")

        with self.assertRaises(AuthenticationError):
            marvelous.api(public_key="Something")

        m = None
        try:
            m = marvelous.api(public_key="Something", private_key="Else")
        except Exception as exc:
            self.fail("marvelous.api() raised {} unexpectedly!".format(exc))

        self.assertEquals(
            m.__class__.__name__, marvelous.session.Session.__name__)

if __name__ == '__main__':
    unittest.main()



from marshmallow import Schema, fields, pre_load, post_load, INCLUDE


class Events():
    def __init__(self, id=None, name=None, **kwargs):
        self.id = id
        self.name = name
        self.unknown = kwargs


class EventsSchema(Schema):
    id = fields.Str()
    name = fields.Str()

    class Meta:
        unknown = INCLUDE

    @pre_load
    def process_input(self, data, **kwargs):
        data['id'] = data['resourceURI'].split('/')[-1]
        return data

    @post_load
    def make(self, data, **kwargs):
        return Events(**data)



import os
import unittest

import marvelous


class TestComics(unittest.TestCase):
    def setUp(self):
        pub = os.getenv('PUBLIC_KEY', 'pub')
        priv = os.getenv('PRIVATE_KEY', 'priv')
        self.m = marvelous.api(
            public_key=pub, private_key=priv,
            cache=marvelous.SqliteCache("tests/testing_mock.sqlite"))

    def test_pulls_verbose(self):
        week = self.m.comics({
            'format': "comic",
            'formatType': "comic",
            'noVariants': True,
            'dateDescriptor': "thisWeek"})

        self.assertTrue(len(week.comics) > 0)

    def test_pulls_simple(self):
        week = self.m.comics({'dateDescriptor': "thisWeek"})
        self.assertTrue(len(week.comics) > 0)

    def test_pulls_simpler(self):
        week = self.m.comics()
        self.assertTrue(len(week.comics) > 0)

    def test_known_comic(self):
        af15 = self.m.comics({'digitalId': 4746}).comics[0]
        self.assertTrue(af15.title == "Amazing Fantasy (1962) #15")
        self.assertTrue(af15.issue_number == 15)
        self.assertTrue(af15.description is None)
        self.assertTrue(af15.format == "Comic")
        self.assertTrue(af15.page_count == 36)
        self.assertTrue(af15.id == 16926)

    def test_invalid_isbn(self):
        """Sometimes Marvel API sends number for isbn"""
        murpg = self.m.comics({'id': 1143}).comics[0]
        self.assertEqual(murpg.isbn, '785110283')

    def test_invalid_diamond_code(self):
        """Sometimes Marvel API sends number for diamond code"""
        hulk = self.m.comics({'id': 27399}).comics[0]
        self.assertEqual(hulk.diamond_code, '0')

if __name__ == '__main__':
    unittest.main()



from marshmallow import Schema, fields, pre_load, post_load, INCLUDE


class Urls():
    def __init__(
            self, digital_purchase_date=None, foc_date=None, onsale_date=None,
            unlimited_date=None, **kwargs):
        self.digital_purchase_date = digital_purchase_date
        self.foc_date = foc_date
        self.onsale_date = onsale_date
        self.unlimited_date = unlimited_date
        self.unknown = kwargs


class UrlsSchema(Schema):
    digitalPurchaseDate = fields.Url(attribute='digital_purchase_date')
    focDate = fields.Url(attribute='foc_date')
    onsaleDate = fields.Url(attribute='onsale_date')
    unlimitedDate = fields.Url(attribute='unlimited_date')

    class Meta:
        unknown = INCLUDE

    @pre_load
    def process_input(self, data, **kwargs):
        new_data = {}
        for d in data:
            new_data[d['type']] = d['url']

        return new_data

    @post_load
    def make(self, data, **kwargs):
        return Urls(**data)



import os
import unittest

import marvelous
from marvelous.comics_list import ComicsList


class TestSeries(unittest.TestCase):
    def setUp(self):
        pub = os.getenv('PUBLIC_KEY', 'pub')
        priv = os.getenv('PRIVATE_KEY', 'priv')
        self.m = marvelous.api(
            public_key=pub, private_key=priv,
            cache=marvelous.SqliteCache("tests/testing_mock.sqlite"))

    def test_known_series(self):
        usms = self.m.series(466)
        self.assertTrue(usms.title == "Ultimate Spider-Man (2000 - 2009)")
        self.assertTrue(usms.id == 466)
        comics = usms.comics()
        self.assertTrue(23931 in [c.id for c in comics])

        self.assertTrue(len(comics[:5]) == 5)
        self.assertTrue(len(comics) == len([x for x in comics if x.id > 3]))

    def test_bad_series(self):
        with self.assertRaises(marvelous.exceptions.ApiError):
            self.m.series(-1)

    def test_bad_response_data(self):
        with self.assertRaises(marvelous.exceptions.ApiError):
            ComicsList({'data': {'results': [{'modified': 'potato'}]}})

    def test_pulls_verbose(self):
        series = self.m.series_list({
            'orderBy': 'modified',
        })

        self.assertGreater(len(series.series), 0)

if __name__ == '__main__':
    unittest.main()



from marshmallow import Schema, fields, pre_load, post_load, INCLUDE

from . import dates, series, urls, events


class Comic():
    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)


class ComicSchema(Schema):
    id = fields.Int()
    digitalId = fields.Int(attribute='digital_id')
    title = fields.Str()
    issueNumber = fields.Int(attribute='issue_number')
    variantDescription = fields.Str(attribute='variant_description')
    description = fields.Str(allow_none=True)
    modified = fields.DateTime()
    isbn = fields.Str()
    up = fields.Str()
    diamondCode = fields.Str(attribute='diamond_code')
    ean = fields.Str()
    issn = fields.Str()
    format = fields.Str()
    pageCount = fields.Int(attribute='page_count')
    # textObjects
    resourceURI = fields.Url(attribute='resource_uri')
    urls = fields.Nested(urls.UrlsSchema)
    series = fields.Nested(series.SeriesSchema)
    # variants
    # collections
    # collectedIssues
    dates = fields.Nested(dates.DatesSchema)
    # prices
    # thumbnail
    images = fields.List(fields.Url)
    # creators
    # characters
    # stories
    events = fields.Nested(events.EventsSchema, many=True)

    class Meta:
        unknown = INCLUDE

    @pre_load
    def process_input(self, data, **kwargs):
        new_data = data

        # Marvel comic 1768, and maybe others, returns a modified of
        # "-0001-11-30T00:00:00-0500". The best way to handle this is
        # probably just to ignore it, since I don't know how to fix it.
        if new_data.get('modified', ' ')[0] == '-':
            del new_data['modified']

        if 'events' in new_data:
            new_data['events'] = new_data['events']['items']

        if 'images' in new_data:
            new_data['images'] = [
                '{}.{}'.format(img['path'], img['extension'])
                for img in new_data['images']
            ]

        if 'isbn' in new_data:
            new_data['isbn'] = str(new_data['isbn'])

        if 'diamondCode' in new_data:
            new_data['diamondCode'] = str(new_data['diamondCode'])

        return new_data

    @post_load
    def make(self, data, **kwargs):
        return Comic(**data)



from . import session, exceptions
from .sqlite_cache import SqliteCache  # NOQA F401 Imported for export


def api(public_key=None, private_key=None, cache=None):
    if public_key is None:
        raise exceptions.AuthenticationError("Missing public_key.")

    if private_key is None:
        raise exceptions.AuthenticationError("Missing private_key.")

    return session.Session(public_key, private_key, cache=cache)



from setuptools import setup, find_packages

version = '1.1.1'

setup(
    author_email='robert@rkuykendall.com',
    author='Robert Kuykendall',
    classifiers=[
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python",
        (
            "Topic :: Internet :: WWW/HTTP :: Dynamic Content "
            ":: CGI Tools/Libraries"
        ),
        "Topic :: Utilities",
    ],
    description="Marvel API wrapper for python.",
    install_requires=['marshmallow', 'requests'],
    keywords='marvel api comics python',
    long_description=open("README.rst", "r").read(),
    name='marvelous',
    packages=find_packages(),
    url='http://github.com/rkuykendall/marvelous',
    version=version,
    zip_safe=True,
)



import datetime
import hashlib
import requests
import urllib.parse

from collections import OrderedDict

from marshmallow import ValidationError

from . import exceptions, comics_list, series, series_list


class Session():
    api_url = "http://gateway.marvel.com:80/v1/public/{}"

    def __init__(self, public_key, private_key, cache=None):

        self.public_key = public_key
        self.private_key = private_key
        self.cache = cache

    def call(self, endpoint, params=None):
        if params is None:
            params = {}

        # Generate part of cache key before hash, apikey and timestamp added
        cache_params = ''
        if len(params) > 0:
            orderedParams = OrderedDict(sorted(params.items(), key=lambda t: t[0]))
            cache_params = '?{}'.format(urllib.parse.urlencode(orderedParams))

        now_string = datetime.datetime.now().strftime('%Y-%m-%d%H:%M:%S')
        auth_hash = hashlib.md5()
        auth_hash.update(now_string.encode('utf-8'))
        auth_hash.update(self.private_key.encode('utf-8'))
        auth_hash.update(self.public_key.encode('utf-8'))

        params['hash'] = auth_hash.hexdigest()
        params['apikey'] = self.public_key
        params['ts'] = now_string

        url = self.api_url.format('/'.join([str(e) for e in endpoint]))
        cache_key = '{url}{cache_params}'.format(
            url=url, cache_params=cache_params)

        if self.cache:
            try:
                cached_response = self.cache.get(cache_key)

                if cached_response is not None:
                    return cached_response
            except AttributeError as e:
                raise exceptions.CacheError(
                    "Cache object passed in is missing attribute: {}".format(
                        repr(e)))

        response = requests.get(url, params=params)

        data = response.json()

        if 'message' in data:
            raise exceptions.ApiError(data['message'])

        if self.cache and response.status_code == 200:
            try:
                self.cache.store(cache_key, data)
            except AttributeError as e:
                raise exceptions.CacheError(
                    "Cache object passed in is missing attribute: {}".format(
                        repr(e)))

        return data

    def comics(self, params=None):
        if params is None:
            params = {}

        return comics_list.ComicsList(
            self.call(['comics'], params=params))

    def series(self, _id):
        try:
            result = series.SeriesSchema().load(self.call(['series', _id]))
        except ValidationError as error:
            raise exceptions.ApiError(error)

        result.session = self
        return result

    def series_list(self, params=None):
        if params is None:
            params = {}

        return series_list.SeriesList(
            self.call(['series'], params=params))


from marshmallow import Schema, fields, pre_load, post_load, INCLUDE

from . import exceptions


class Series:
    def __init__(self, **kwargs):
        if 'response' not in kwargs:
            kwargs['response'] = None

        for k, v in kwargs.items():
            if k == "comics":
                continue
            setattr(self, k, v)

    def comics(self, params=None):
        from . import comics_list

        if params is None:
            params = {}

        return comics_list.ComicsList(
            self.session.call(['series', self.id, 'comics'], params=params))


class SeriesSchema(Schema):
    response = fields.Raw()
    id = fields.Int()
    resourceURI = fields.Str(attribute='resource_uri')
    title = fields.Str()

    class Meta:
        unknown = INCLUDE

    @pre_load
    def process_input(self, data, **kwargs):
        if data.get('code', 200) != 200:
            raise exceptions.ApiError(data.get('status'))

        if 'status' in data:
            data['data']['results'][0]['response'] = data
            data = data['data']['results'][0]

        data['id'] = data['resourceURI'].split('/')[-1]
        return data

    @post_load
    def make(self, data, **kwargs):
        return Series(**data)



import json
import os
import requests_mock
import unittest

import marvelous


class NoGet:
    def store(self, key, value):
        # This method should store key value pair
        return


class NoStore:
    def get(self, key):
        return None


class TestCache(unittest.TestCase):
    def setUp(self):
        self.pub = os.getenv('PUBLIC_KEY', 'pub')
        self.priv = os.getenv('PRIVATE_KEY', 'priv')

    def test_no_get(self):
        m = marvelous.api(
            public_key=self.pub, private_key=self.priv,
            cache=NoGet())

        with self.assertRaises(marvelous.exceptions.CacheError):
            m.series(466)

    def test_no_store(self):
        m = marvelous.api(
            public_key=self.pub, private_key=self.priv,
            cache=NoStore())

        with requests_mock.Mocker() as r:
            r.get(
                'http://gateway.marvel.com:80/v1/public/series/466',
                text='{"response_code": 200}')

            with self.assertRaises(marvelous.exceptions.CacheError):
                m.series(466)

    def test_sql_store(self):
        fresh_cache = marvelous.SqliteCache(":memory:")
        test_cache = marvelous.SqliteCache("tests/testing_mock.sqlite")

        m = marvelous.api(
            public_key=self.pub, private_key=self.priv,
            cache=fresh_cache)
        url = 'http://gateway.marvel.com:80/v1/public/series/466'

        self.assertTrue(fresh_cache.get(url) is None)

        try:
            with requests_mock.Mocker() as r:
                r.get(url, text=json.dumps(test_cache.get(url)))
                m.series(466)

            self.assertTrue(fresh_cache.get(url) is not None)
        except TypeError:
            print(
                'This test will fail after cache db deleted.\n'
                'It should pass if you now re-run the test suite without deleting the database.')
            assert(False)


if __name__ == '__main__':
    unittest.main()



from marshmallow import Schema, fields, pre_load, post_load, INCLUDE


class Dates():
    def __init__(self, on_sale=None, foc=None, unlimited=None, **kwargs):
        self.on_sale = on_sale
        self.foc = foc
        self.unlimited = unlimited
        self.unknown = kwargs


class DatesSchema(Schema):
    onsaleDate = fields.DateTime(attribute='on_sale')
    focDate = fields.DateTime(attribute='foc')
    unlimitedDate = fields.DateTime(attribute='unlimited')

    class Meta:
        unknown = INCLUDE

    @pre_load
    def process_input(self, data, **kwargs):
        new_data = {}
        for d in data:
            # Marvel comic 4373, and maybe others, returns a focDate of
            # "-0001-11-30T00:00:00-0500". The best way to handle this is
            # probably just to ignore it, since I don't know how to fix it.
            if d['date'][0] != '-':
                new_data[d['type']] = d['date']

        return new_data

    @post_load
    def make(self, data, **kwargs):
        return Dates(**data)
